@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
 
-from .mysolr_response import SolrResponse
+from .response import SolrResponse
 from .mysolr import Solr
 from .mysolr import Cursor
 
 __title__ = 'mysolr'
-__version__ = '0.6.1'
-__build__ = 0x000061
+__version__ = '0.7'
+__build__ = 0x000070
 __author__ = 'Rubén Abad, Miguel Olivares'
